Attribute VB_Name = "UTMconv"
Option Explicit
Const PI = 3.14159265358979
Const deg2rad = PI / 180
Const rad2deg = 180 / PI

Public Sub LLtoUTM(ByVal LAT As Double, ByVal LON As Double, _
ByRef UTMNorthing As Double, ByRef UTMEasting As Double, UTMZone)

'converts lat/long to UTM coords.  Equations from USGS Bulletin 1532
'East Longitudes are positive, West longitudes are negative.
'North latitudes are positive, South latitudes are negative
'Lat and Long are in decimal degrees
'Written by Chuck Gantz- chuck.gantz@globalstar.com
'Translated into VB by Herko@t-online.de Upmann
Dim LongOrigin As Double
Dim eccPrimeSquared As Double
Dim n As Double, T As Double, C As Double, a As Double, M As Double
Dim eccSquared As Double, k0 As Double
Dim LongTemp As Double
Dim LatRad As Double
Dim LongRad As Double
Dim LongOriginRad As Double
Dim ZoneNumber As Integer
Dim elli As Double

    
elli = 6378137 'equatorial radius for WGS-84
eccSquared = 0.00669438 'excentricity for WGS-84
k0 = 0.9996
'Make sure the longitude is between -180.00 .. 179.9
LongTemp = (LON + 180) - Int((LON + 180) / 360) * 360 - 180 ' -180.00 .. 179.9;
LatRad = LAT * deg2rad
LongRad = LongTemp * deg2rad
       

ZoneNumber = CInt((LongTemp + 180) / 6) + 1
  
If ((LAT >= 56) And (LAT < 64) And (LongTemp >= 3) And (LongTemp < 12)) Then ZoneNumber = 32

  ' Special zones for Svalbard
If ((LAT >= 72) And (LAT < 84)) Then
    If ((LongTemp >= 0) And (LongTemp < 9)) Then
        ZoneNumber = 31
    ElseIf ((LongTemp >= 9) And (LongTemp < 21)) Then ZoneNumber = 33
    ElseIf ((LongTemp >= 21) And (LongTemp < 33)) Then ZoneNumber = 35
    ElseIf ((LongTemp >= 33) And (LongTemp < 42)) Then ZoneNumber = 37
    End If
End If
 
LongOrigin = (ZoneNumber - 1) * 6 - 180 + 3 '+3 puts origin in middle of zone
LongOriginRad = LongOrigin * deg2rad

'compute the UTM Zone from the latitude and longitude
'sprintf(UTMZone, "%d%c", ZoneNumber, UTMLetterDesignator(Lat));
UTMZone = ZoneNumber & UTMLetterDesignator(LAT)

eccPrimeSquared = (eccSquared) / (1 - eccSquared)
n = elli / Sqr(1 - eccSquared * Sin(LatRad) * Sin(LatRad))
T = Tan(LatRad) * Tan(LatRad)
C = eccPrimeSquared * Cos(LatRad) * Cos(LatRad)
a = Cos(LatRad) * (LongRad - LongOriginRad)

M = elli * ((1 - eccSquared / 4 - 3 * eccSquared * eccSquared / 64 - 5 * eccSquared * eccSquared * eccSquared / 256) * LatRad _
        - (3 * eccSquared / 8 + 3 * eccSquared * eccSquared / 32 + 45 * eccSquared * eccSquared * eccSquared / 1024) * Sin(2 * LatRad) _
        + (15 * eccSquared * eccSquared / 256 + 45 * eccSquared * eccSquared * eccSquared / 1024) * Sin(4 * LatRad) _
        - (35 * eccSquared * eccSquared * eccSquared / 3072) * Sin(6 * LatRad))

UTMEasting = CDbl(k0 * n * (a + (1 - T + C) * a * a * a / 6 _
                + (5 - 18 * T + T * T + 72 * C - 58 * eccPrimeSquared) * a * a * a * a * a / 120) _
                + 500000)

UTMNorthing = CDbl(k0 * (M + n * Tan(LatRad) * (a * a / 2 + (5 - T + 9 * C + 4 * C * C) * a * a * a * a / 24 _
             + (61 - 58 * T + T * T + 600 * C - 330 * eccPrimeSquared) * a * a * a * a * a * a / 720)))

If (LAT < 0) Then UTMNorthing = UTMNorthing + 10000000  '10000000 meter offset for southern hemisphere

End Sub

Private Function UTMLetterDesignator(LAT As Double) As String


'This routine determines the correct UTM letter designator for the given latitude
'returns 'Z' if latitude is outside the UTM limits of 84N to 80S
'Written by Chuck Gantz- chuck.gantz@globalstar.com
Dim LetterDesignator As String

If ((84 >= LAT) And (LAT >= 72)) Then
    LetterDesignator = "X"
ElseIf ((72 > LAT) And (LAT >= 64)) Then LetterDesignator = "W"
ElseIf ((64 > LAT) And (LAT >= 56)) Then LetterDesignator = "V"
ElseIf ((56 > LAT) And (LAT >= 48)) Then LetterDesignator = "U"
ElseIf ((48 > LAT) And (LAT >= 40)) Then LetterDesignator = "T"
ElseIf ((40 > LAT) And (LAT >= 32)) Then LetterDesignator = "S"
ElseIf ((32 > LAT) And (LAT >= 24)) Then LetterDesignator = "R"
ElseIf ((24 > LAT) And (LAT >= 16)) Then LetterDesignator = "Q"
ElseIf ((16 > LAT) And (LAT >= 8)) Then LetterDesignator = "P"
ElseIf ((8 > LAT) And (LAT >= 0)) Then LetterDesignator = "N"
ElseIf ((0 > LAT) And (LAT >= -8)) Then LetterDesignator = "M"
ElseIf ((-8 > LAT) And (LAT >= -16)) Then LetterDesignator = "L"
ElseIf ((-16 > LAT) And (LAT >= -24)) Then LetterDesignator = "K"
ElseIf ((-24 > LAT) And (LAT >= -32)) Then LetterDesignator = "J"
ElseIf ((-32 > LAT) And (LAT >= -40)) Then LetterDesignator = "H"
ElseIf ((-40 > LAT) And (LAT >= -48)) Then LetterDesignator = "G"
ElseIf ((-48 > LAT) And (LAT >= -56)) Then LetterDesignator = "F"
ElseIf ((-56 > LAT) And (LAT >= -64)) Then LetterDesignator = "E"
ElseIf ((-64 > LAT) And (LAT >= -72)) Then LetterDesignator = "D"
ElseIf ((-72 > LAT) And (LAT >= -80)) Then LetterDesignator = "C"
Else
LetterDesignator = "Z" 'This is here as an error flag to show that the Latitude is outside the UTM limits
End If
UTMLetterDesignator = LetterDesignator
End Function


Public Sub UTMtoLL(UTMNorthing As Double, UTMEasting As Double, UTMZone, _
              ByRef LAT As Double, ByRef LON As Double)

'converts UTM coords to lat/long.  Equations from USGS Bulletin 1532
'East Longitudes are positive, West longitudes are negative.
'North latitudes are positive, South latitudes are negative
'Lat and Long are in decimal degrees.
'Written by Chuck Gantz- chuck.gantz@globalstar.com
'Translated into VB by Herko@t-online.de Upmann

Dim k0 As Double, a As Double, eccSquared As Double, eccPrimeSquared As Double
Dim e1 As Double
Dim N1 As Double, T1 As Double, C1 As Double, R1 As Double, D As Double, M As Double
Dim LongOrigin As Double, mu As Double, phi1 As Double, phi1Rad As Double
Dim x As Double, y As Double
Dim ZoneNumber As Integer
Dim ZoneLetter
Dim NorthernHemisphere As Integer '1 for northern hemisphere, 0 for southern



a = 6378137 'equatorial radius for WGS-84
eccSquared = 0.00669438 'excentricity for WGS-84
k0 = 0.9996

e1 = (1 - Sqr(1 - eccSquared)) / (1 + Sqr(1 - eccSquared))

x = UTMEasting - 500000   'remove 500,000 meter offset for longitude
y = UTMNorthing

'ZoneNumber = strtoul(UTMZone, &ZoneLetter, 10)
ZoneNumber = 32
'if((*ZoneLetter - 'N') >= 0)
NorthernHemisphere = 1 'point is in northern hemisphere
'Else
'NorthernHemisphere = 0;'point is in southern hemisphere
'y -= 10000000.0;'remove 10,000,000 meter offset used for southern hemisphere

LongOrigin = (ZoneNumber - 1) * 6 - 180 + 3 '+3 puts origin in middle of zone
eccPrimeSquared = (eccSquared) / (1 - eccSquared)
M = y / k0
mu = M / (a * (1 - eccSquared / 4 - 3 * eccSquared * eccSquared / 64 - 5 * eccSquared * eccSquared * eccSquared / 256))
phi1Rad = mu + (3 * e1 / 2 - 27 * e1 * e1 * e1 / 32) * Sin(2 * mu) _
                + (21 * e1 * e1 / 16 - 55 * e1 * e1 * e1 * e1 / 32) * Sin(4 * mu) _
                + (151 * e1 * e1 * e1 / 96) * Sin(6 * mu)
phi1 = phi1Rad * rad2deg

N1 = a / Sqr(1 - eccSquared * Sin(phi1Rad) * Sin(phi1Rad))
T1 = Tan(phi1Rad) * Tan(phi1Rad)
C1 = eccPrimeSquared * Cos(phi1Rad) * Cos(phi1Rad)
R1 = a * (1 - eccSquared) / (1 - eccSquared * Sin(phi1Rad) * Sin(phi1Rad)) ^ 1.5
D = x / (N1 * k0)

LAT = phi1Rad - (N1 * Tan(phi1Rad) / R1) * (D * D / 2 - (5 + 3 * T1 + 10 * C1 - 4 * C1 * C1 - 9 * eccPrimeSquared) * D * D * D * D / 24 _
    + (61 + 90 * T1 + 298 * C1 + 45 * T1 * T1 - 252 * eccPrimeSquared - 3 * C1 * C1) * D * D * D * D * D * D / 720)
LAT = LAT * rad2deg

LON = (D - (1 + 2 * T1 + C1) * D * D * D / 6 + (5 - 2 * C1 + 28 * T1 - 3 * C1 * C1 + 8 * eccPrimeSquared + 24 * T1 * T1) _
                    * D * D * D * D * D / 120) / Cos(phi1Rad)
LON = LongOrigin + LON * rad2deg

End Sub

' Converts a decimal degree number like 57,123456 into a double containing
' the separated DgrMinSec values e.g. "570724,4416"
Public Function GradDezitoGrad(ByVal Grdz As Double) As Double
Dim dgr As Double
Dim Min As Double
Dim val As Double

dgr = Int(Grdz)
val = (Grdz - dgr) * 60
Min = Int(val)
val = (val - Min) * 60
GradDezitoGrad = val + Min * 100 + dgr * 10000

End Function

' Converts a decimal degree number like 57,123456 into a string containing
' the separated DgrMinSec values e.g. "570724,4416"
Public Function GradDezitoString(ByVal Grdz As Double) As String

Dim Dstring As String, Mstring As String, Sstring As String

Dim dgr As Double
Dim Min As Double
Dim val As Double

dgr = Int(Grdz)
val = (Grdz - dgr) * 60
Min = Int(val)
val = (val - Min) * 60

Dstring = Format(dgr, "000")
Mstring = Format(Min, "00")
Sstring = Format(val, "00.000")
Mid(Sstring, Len(Sstring) - 3, 1) = "." ' convert comma into point
GradDezitoString = Dstring + Mstring + Sstring

End Function

Public Function CommaCnv(ByVal val As Double) As String
Dim ts As String
Dim length As Integer
Dim n As Integer

ts = Format(val, "###.########")
length = Len(ts)
For n = 1 To length
    If Mid(ts, n, 1) = "," Then
        Mid(ts, n, 1) = "."
    End If
Next n
CommaCnv = ts
End Function
